"""Flask extension that proxies requests to Frontegg."""

import os
import typing
from urllib.parse import urljoin

import requests
from flask import (Blueprint, Flask, Request, current_app, make_response,
                   request)
from flask_cors import cross_origin
from werkzeug.datastructures import Headers

from frontegg._mixins import AuditsClientMixin
from frontegg.client import BaseFronteggClient
from frontegg.permissions import ForbiddenRequest


class FronteggFlaskClient(BaseFronteggClient[Request]):
    @property
    def current_request(self) -> Request:
        return request

    @property
    def base_url(self) -> str:
        return current_app.config['FRONTEGG_API_GATEWAY_URL']

    @property
    def authentication_service_url(self):
        return current_app.config['FRONTEGG_AUTHENTICATION_SERVICE_URL']

    def proxy_request(self, endpoint: str) -> requests.Response:
        """Proxy a request from the client to Frontegg's API.

        :param endpoint: The endpoint to perform the request to.
        :return: The response of the request.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        """
        if request.method.upper() == 'GET':
            data = request.args
        else:
            data = request.json
        return self.request(endpoint, request.method, data=data)


class Frontegg(AuditsClientMixin):
    """Frontegg Flask Extension.

    >>> from flask import Flask
    >>> app.config['FRONTEGG_CLIENT_ID'] = '00000000-0000-0000-0000-000000000000'
    >>> app.config['FRONTEGG_API_KEY'] = 'api-key'
    >>> app.config['FRONTEGG_CONTEXT_RESOLVER'] = lambda: ('me@example.com', 'tenant-id')
    >>> app = Flask(__name__)
    >>> frontegg = Frontegg(app)
    """

    def __init__(self, app: typing.Optional[Flask] = None):
        """Initialize the Frontegg extension.

        If an app is provided, then the extension is registered to the app.
        Otherwise, call :meth:`frontegg.Frontegg.init_app`.

        :param app: The flask application to extend.
        """
        try:
            client_id = app.config['FRONTEGG_CLIENT_ID']
            api_key = app.config['FRONTEGG_API_KEY']
            context_callback = app.config['FRONTEGG_CONTEXT_RESOLVER']
        except KeyError as e:
            raise ValueError(
                "{} must be specified in the application's configuration.".format(e))

        self._client = FronteggFlaskClient(
            client_id, api_key, context_callback)

        if app is not None:
            self.init_app(app)

    @property
    def _config(self) -> dict:
        """The current flask app's configuration"""
        return current_app.config

    def init_app(self, app: Flask) -> None:
        """Initialize the extension for the app.

        :param app: The flask application to extend.
        """
        frontegg_api_gateway_url = os.environ.get(
            'FRONTEGG_API_GATEWAY_URL', 'https://api.frontegg.com/')
        app.config.setdefault('FRONTEGG_API_GATEWAY_URL',
                              frontegg_api_gateway_url)

        frontegg_authentication_service_url = os.environ.get('FRONTEGG_AUTHENTICATION_SERVICE_URL',
                                                             urljoin(frontegg_api_gateway_url,
                                                                     '/vendors/auth/token'))
        app.config.setdefault(
            'FRONTEGG_AUTHENTICATION_SERVICE_URL', frontegg_authentication_service_url)

        frontegg_audits_service_url = os.environ.get('FRONTEGG_AUDITS_SERVICE_URL',
                                                     urljoin(frontegg_api_gateway_url,
                                                             '/audits'))
        app.config.setdefault('FRONTEGG_AUDITS_SERVICE_URL',
                              frontegg_audits_service_url)

        frontegg_metadata_service_url = os.environ.get('FRONTEGG_METADATA_SERVICE_URL',
                                                       urljoin(frontegg_api_gateway_url,
                                                               '/metadata'))
        app.config.setdefault('FRONTEGG_METADATA_SERVICE_URL',
                              frontegg_metadata_service_url)

        frontegg_blueprint = Blueprint(
            'frontegg', __name__, url_prefix='/frontegg')

        @frontegg_blueprint.route('/<path:endpoint>',
                                  methods=('GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'HEAD', 'OPTIONS'))
        @cross_origin()
        def proxy(endpoint):
            proxy_response = self._client.proxy_request(endpoint)

            try:
                response = make_response(
                    proxy_response.content, proxy_response.status_code)
                response_headers = {
                    **response.headers,
                    **{key.title(): value for key, value in proxy_response.headers.items()}
                }
                response.headers = Headers(response_headers)
                response.headers.pop('Access-Control-Allow-Origin')
                return response
            except ForbiddenRequest:
                return {'message': "Forbidden"}, 403

        app.register_blueprint(frontegg_blueprint)


__all__ = ('Frontegg',)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['frontegg']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=0.15.5,<0.16.0', 'requests>=2.22.0,<3.0.0']

extras_require = \
{'flask': ['flask>=1.0,<2.0', 'flask-cors>=3.0,<4.0']}

setup_kwargs = {
    'name': 'frontegg',
    'version': '0.1.3',
    'description': 'Frontegg is a web platform where SaaS companies can set up their fully managed, scalable and brand aware - SaaS features and integrate them into their SaaS portals in up to 5 lines of code.',
    'long_description': ".. image:: https://fronteggstuff.blob.core.windows.net/frongegg-logos/logo-transparent.png\n   :alt: Frontegg\n\nFrontegg is a web platform where SaaS companies can set up their fully managed, scalable and brand aware - SaaS features and integrate them into their SaaS portals in up to 5 lines of code.\n\nInstallation\n------------\n\nYou can install this library by using pip::\n\n    pip install frontegg\n\nIf you'd like to use Frontegg in your flask project use the flask extra dependency::\n\n    pip install frontegg[flask]\n\nUsage\n-----\n\nYou can directly use the Frontegg REST client to access your data when you need to.\nThe raw client directly returns a `requests.Response`.\n\nThis is useful in Python scripts or in cases where you need to access the data from the server side:\n\n.. code-block:: pycon\n\n    >>> from frontegg import FronteggClient\n    >>> client = FronteggClient('your-client-id', 'your-client-secret')\n    >>> response = client.request('/metadata', 'GET', data={'entityName': 'audits'})\n    >>> print(response.json())\n    {'rows': [{'_id': '5d3d2ee54a04a50033da91df', 'entityName': 'audits', 'properties': [\n        {'_id': '5d3d2ee54a04a50033da91e6', 'name': 'createdAt', 'displayName': 'Date', 'type': 'Timestamp',\n         'filterable': True, 'sortable': True},\n        {'_id': '5d3d2ee54a04a50033da91e5', 'name': 'user', 'displayName': 'User', 'type': 'UserIdentity',\n         'filterable': True, 'sortable': True},\n        {'_id': '5d3d2ee54a04a50033da91e4', 'name': 'resource', 'displayName': 'Resource', 'type': 'AlphaNumeric',\n         'filterable': True, 'sortable': True},\n        {'_id': '5d3d2ee54a04a50033da91e3', 'name': 'action', 'displayName': 'Action', 'type': 'AlphaNumeric',\n         'filterable': True, 'sortable': True}, {\n            '_id': '5d3d2ee54a04a50033da91e2', 'name': 'severity', 'displayName': 'Severity', 'type': 'AlphaNumeric',\n            'filterable': True, 'sortable': True},\n        {'_id': '5d3d2ee54a04a50033da91e1', 'name': 'ip', 'displayName': 'IP Address', 'type': 'AlphaNumeric',\n         'filterable': True, 'sortable': True},\n        {'_id': '5d3d2ee54a04a50033da91e0', 'name': 'message', 'displayName': 'Message', 'type': 'AlphaNumeric',\n         'filterable': True, 'sortable': False}], 'vendorId': '2fb835d0-8f17-48cb-8489-3a2f8f50ce4a',\n                          'id': '39372f0f-1d14-4ecd-8462-1b22d5ca9264', 'createdAt': '2019-07-28T05:13:09.723Z',\n                          'updatedAt': '2019-07-28T05:13:09.723Z', '__v': 0}]}\n\nYou can pass a context callback to provide the user id and the tenant id that are relevant to the request:\n\n.. code-block:: pycon\n\n    >>> from frontegg import FronteggClient, FronteggContext\n    >>> client = FronteggClient('your-client-id', 'your-client-secret', context_callback=lambda request: FronteggContext('user_id@user.com', 'tenant-id'))\n    >>> response = client.request('/audits/stats', 'GET', data={'tenantId': 'tenant-id'})\n    >>> print(response.json())\n    {'totalToday': 2, 'severeThisWeek': 0}\n\nNote that when using the client directly, the request argument is always `None` since you are performing the request\ndirectly.\n\nProxying Requests From Other Clients\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nThe browser and possibly other clients in your architecture need to access Frontegg from your proxy\nas it determines the context of the request.\n\nThis package currently only supports doing so from Flask. In the future, more frameworks may be added.\n\nFlask\n+++++\n\nTo proxy requests to Frontegg in your Flask application the following configuration must be provided:\n\n.. code-block:: python\n\n    from flask import Flask\n    app = Flask('example')\n    app.config['FRONTEGG_CLIENT_ID'] = '2fb835d0-8f17-48cb-8489-3a2f8f50ce4a'\n    app.config['FRONTEGG_API_KEY'] = 'my-api-key'\n\nYou can optionally provide the context callback as a configuration setting as well:\n\n.. code-block:: python\n\n    from frontegg import FronteggContext\n    app.config['FRONTEGG_CONTEXT_RESOLVER'] = lambda request: FronteggContext('user_id@user.com', 'my-tenant-id')\n\nThe request argument will be filled with the `flask.request` object.\nYou can use it to determine the user id and the tenant id.\n\nIn addition, different users may or may not access specific data and thus have different permissions.\nYou can use the `frontegg.FronteggPermissions` enum to limit access to your data.\n\n.. code-block:: python\n\n    from frontegg import FronteggContext, FronteggPermissions\n\n    def context_resolver(request):\n        if is_admin_user(request):\n            permissions = (FronteggPermissions.All,)\n        else:\n            permissions = (FronteggPermissions.Teams.value.Read,)\n\n        return FronteggContext('user_id@user.com', 'my-tenant-id', permissions=permissions)\n\n\nTo begin proxying requests you should\n\n.. code-block:: python\n\n    from flask import Flask\n    from frontegg import FronteggContext\n    from frontegg.flask import Frontegg\n    app = Flask('example')\n    app.config['FRONTEGG_CLIENT_ID'] = '2fb835d0-8f17-48cb-8489-3a2f8f50ce4a'\n    app.config['FRONTEGG_API_KEY'] = 'my-api-key'\n    app.config['FRONTEGG_CONTEXT_RESOLVER'] = lambda request: FronteggContext('user_id@user.com', 'my-tenant-id')\n    frontegg = Frontegg(app)\n\nYou can use the Frontegg object to access your audits data. Please refer to the documentation for more details.",
    'author': 'Omer Katz',
    'author_email': 'omer.drow@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://frontegg.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
